# -----------------------------------------------------------------------
# Custom types/data / simplevm.py
# (c) Hex-Rays
#
import struct
import sys

# -----------------------------------------------------------------------
def swap_dic(d):
    r = {}
    for (k, v) in d.items():
        r[v] = k
    return r

# -----------------------------------------------------------------------
table = {
    'add' : 1, # add reg, reg|imm32
    'mul' : 2, # mul reg, reg|imm32
    'sub' : 3, # sub reg, reg|imm32
    'xor' : 4, # xor reg, reg|imm32
    'mov' : 5  # mov reg, reg|imm32
}

registers = {
    'r1': 1,
    'r2': 2,
    'r3': 3
}

rtable     = swap_dic(table)
rregisters = swap_dic(registers)

# -----------------------------------------------------------------------
def assemble(inst, op1, op2):
    try:
        opcode = table[inst]
    except:
        return (False, "Unknown instruction!")

    try:
        r1 = registers[op1]
        if not r1:
            raise
    except:
        return (False, "op1 must be a register name")

    if isinstance(op2, basestring):
        imm = None
        try:
            r2 = registers[op2]
        except:
            return (False, "op2 must be a register name")
    else:
        try:
            imm = int(op2)
            r2 = 0
        except:
            return (False, "op2 must be a register name")

    s = chr((opcode << 4) | (r1 << 2) | r2)
    if imm is not None:
        imm = struct.pack('L', imm)
        s += imm

    return (True, s)

# -----------------------------------------------------------------------
def asms(lines):
    code = []
    for line in lines.split('\n'):
        ops = (line.strip()).split(' ')
        if not ops[0]:
            continue

        # not a register? then an immediate
        if ops[2][0] != 'r':
            ops[2] = int(ops[2], 16)
        # assemble line
        ok, s = assemble(*ops)
        if not ok:
            return (False, s)
        code.append(s)
    return "".join(code)

# -----------------------------------------------------------------------
def disasm(inst):
    opbyte = ord(inst[0])
    op     = opbyte >> 4
    r1     = (opbyte & 0xf) >> 2
    r2     = opbyte & 3
    sz     = 0
    if r2 == 0:
        imm = struct.unpack_from('L', inst, 1)[0]
        sz  = 5
    else:
        imm = None
        sz  = 1
    text = "%s %s, %s" % (rtable[op], rregisters[r1], "0x%08X" % imm if imm is not None else rregisters[r2])
    return (sz, text)

def dasms(inst):
    p = 0
    L = len(inst)
    R = []
    while p < L:
        sz, text = disasm(inst[p:])
        opcode = ",".join(["0x%02x" % ord(x) for x in inst[p:p+sz]])
        R.append((p, text, opcode))
        p += sz
    return R

# -----------------------------------------------------------------------

codes = ("""
mul r1 10
mul r2 5
add r2 r1
xor r1 0xabcd
add r1 r2
mov r3 0xdeadbeef
xor r1 r3
sub r1 r2
mul r2 r3
add r1 r2"""
,
"""
mov r1 1
mov r2 5
mul r2 2
add r1 r2
"""
,
"""
xor r2 r3
add r1 r3
add r1 r2
""",
"""
mul r1 r1
mul r2 r2
add r3 r1
add r3 r2
xor r2 r1
add r3 r2
add r3 0x45811900
mov r1 r3
""")

sep = '-' * 70
for c in codes:
    R = dasms(asms(c))
    print sep
    for (p, text, opcode) in R:
        print "%s, // +%02X: %s" % (opcode, p, text)
